-----------------------------------------------------------------------------------------
-- Start Activity
-----------------------------------------------------------------------------------------

function MenouMission:StartActivity()
    print("START! -- MenouMission:StartActivity()!");
    
    self.FightStage = { NOFIGHT = 0, DEFENDING = 1, ATTACK = 2, FIGHTSTAGECOUNT = 3 };

    self.AreaTimer = Timer();
    self.StepTimer = Timer();
    self.SpawnTimer = Timer();
    self.ScreenChange = false;

    self.CurrentFightStage = self.FightStage.NOFIGHT;
    self.CPUBrain = nil;
	self.alarmSounded = false;
	
	self.playerActor = nil;
	self.playerSpawnClear = false;
	self.timer = 0;
	self.WeaponList = {"Dummy Sniper", "Dummy Grenade Launcher", "Dummy Blaster", "Dummy Nailgun", "Dummy Pistol", "Dummy Digger"};
	self.Lives = 7;

    --------------------------
    -- Set up teams

    -- Team 2 is always CPU
    self.CPUTeam = Activity.TEAM_2;
	ActivityMan:GetActivity():SetTeamFunds(0,0)
    
    --------------------------
    -- Set up players
    
    for player = 0, self.PlayerCount - 1 do
		local team = self:GetTeamOfPlayer(player);

		self.CPUBrain = MovableMan:GetUnassignedBrain(self.CPUTeam);
        -- Check if we already have a brain assigned
        if not self:GetPlayerBrain(player) then
            local foundBrain = MovableMan:GetUnassignedBrain(self:GetTeamOfPlayer(player));
            -- If we can't find an unassigned brain in the scene to give each player, then force to go into editing mode to place one
            if not foundBrain then
                self.ActivityState = Activity.EDITING;
                print("Can't find player brain, forcing player to place one manually in edit mode!");
            else
                -- Set the found brain to be the selected actor at start
                self:SetPlayerBrain(foundBrain, player);
                self:SetLandingZone(self:GetPlayerBrain(player).Pos, player);
                -- Set the observation target to the brain, so that if/when it dies, the view flies to it in observation mode
                self:SetObservationTarget(self:GetPlayerBrain(player).Pos, player)
            end
        end
		
    end
    
    self.AreaTimer:Reset();
    self.StepTimer:Reset();
    self.SpawnTimer:Reset();
    
end

-- Spawn a soldier, give him his gun, and load him into a rocket
function spawnPlayer(player, team, gun)
	local ship = CreateACRocket("Drop Crate", "Dummy.rte");
	local p = CreateAHuman("Soldier Light", "Coalition.rte")
	local gun = CreateHDFirearm("Assault Rifle", "Coalition.rte");
    p:AddInventoryItem(gun);
	p.AIMode = Actor.AIMODE_SENTRY;
	ship:AddInventoryItem(p);
	ship.Team = -1
	p.Team = team
	local deploymentZone = SceneMan.Scene:GetArea("Coalitionlanding");
    ship.Pos = Vector(deploymentZone:GetRandomPoint().X, 100);
	MovableMan:AddActor(ship);
	return p;
end

-----------------------------------------------------------------------------------------
-- Pause Activity
-----------------------------------------------------------------------------------------

function MenouMission:PauseActivity(pause)
    print("PAUSE! -- MenouMission:PauseActivity()!");
end

-----------------------------------------------------------------------------------------
-- End Activity
-----------------------------------------------------------------------------------------

function MenouMission:EndActivity()
    print("END! -- MenouMission:EndActivity()!");
end

-----------------------------------------------------------------------------------------
-- Update Activity-----------------------------------------------------------------------------------------

function MenouMission:UpdateActivity()
    -- Avoid game logic when we're editing
    if (self.ActivityState == Activity.EDITING) then
        self:UpdateEditing();
        return;
    end

	-- Odd need for a delay here, probably to allow actors to spawn
	if self.timer == 50 then
		self.playerSpawnClear = true
		for actor in MovableMan.Actors do
            if actor.Team == self.CPUTeam then
				-- Set all pre-placed enemies to sentry AI mode
                actor:SetControllerMode(Controller.CIM_AI, -1);
                actor.AIMode = Actor.AIMODE_SENTRY;
           end
        end
	end
	self.timer = self.timer + 1;
	
		local player = 0;
        local team = self:GetTeamOfPlayer(player);
		
		-- If there's no player yet, make one
		if ((not MovableMan:IsActor(self.playerActor)) and self.playerSpawnClear) then
			self.playerSpawnClear = false;
			self.playerswitched = false;
			self.Lives = self.Lives - 1;
			if (self.Lives <= 0) then
				-- Game over
				self:GetPlayerBrain(player):GibThis(); -- Gib the brain, just to rub it in
				FrameMan:SetScreenText("Mission Failed", player, 333, -1, false);
				self.WinnerTeam = self:OtherTeam(team);
				ActivityMan:EndActivity(); -- Game over
			else
				-- Clear to make a new player
				self.playerActor = spawnPlayer(player, team, gun);
			end
		end
		if (MovableMan:IsActor(self.playerActor)) then
			self.playerSpawnClear = true;
		end
		
		if (not self.playerswitched and MovableMan:IsActor(self.playerActor)) then
			-- Switch to the new player
			self:SwitchToActor(self.playerActor, player, self:GetTeamOfPlayer(player));
			self.playerswitched = true;
			self:ResetMessageTimer(Activity.PLAYER_1);
            FrameMan:ClearScreenText(Activity.PLAYER_1);
			-- Display life counter
			if (self.Lives == 1) then
				FrameMan:SetScreenText(self.Lives .. " LIFE LEFT!", Activity.PLAYER_1, 0, 1000, false);
			else
				FrameMan:SetScreenText(self.Lives .. " LIVES LEFT", Activity.PLAYER_1, 0, 1000, false);
			end
		end
    
    --------------------------------------------
    -- Battle logic
	
	if self:GetControlledActor(Activity.PLAYER_1) then
		-- Trigger the "alarm"
		if (SceneMan.Scene:WithinArea("Dummybase", self:GetControlledActor(Activity.PLAYER_1).Pos)) then
			self.alarmSounded = true;
		end
	end

	-- Drop in reinforcement periodically once the alarm has been sounded
    if (self.SpawnTimer:IsPastSimMS(25000) and self.alarmSounded) then
        local ship = CreateACDropShip("Dummy Drop Ship");
        local dummy = CreateAHuman("Dummy");
        local gun = CreateHDFirearm(self.WeaponList[math.random(#self.WeaponList)]);
        dummy:AddInventoryItem(gun);
		dummy.AIMode = Actor.AIMODE_PATROL;
        ship:AddInventoryItem(dummy);
        -- See if there is a designated LZ Area for attackers, and only land over it
        local attackLZ = SceneMan.Scene:GetArea("Dummylanding");
        if attackLZ then
            ship.Pos = Vector(attackLZ:GetRandomPoint().X, 0);
        else
            -- Will appear anywhere when there is no designated LZ
            ship.Pos = Vector(SceneMan.Scene.Width * PosRand(), 0);
        end
        ship.Team = self.CPUTeam;
        ship:SetControllerMode(Controller.CIM_AI, -1);
        -- Let the spawn into the world, passing ownership
        MovableMan:AddActor(ship);
        -- Wait another period for next spawn
        self.SpawnTimer:Reset();
    end
	
	if not MovableMan:IsActor(self.CPUBrain) and ActivityMan:ActivityRunning() then
        self.CPUBrain = nil;
        -- Proclaim player winner end
        self.WinnerTeam = Activity.TEAM_1
		FrameMan:SetScreenText("Mission Accomplished! Good work soldier!", player, 0, -1, false);
        -- Finito!
        ActivityMan:EndActivity();
    end

end